#!/usr/bin/env python3

import os
import yaml
import typing


class Config(object):
    _CONFIG_FILE: typing.Optional[str] = None
    _CONFIG: typing.Optional[dict] = None

    # define required attributes
    _GLOBAL: typing.Optional[str] = None
    _BORGBACKUP: typing.Optional[str] = None
    _RESTIGBACKUP: typing.Optional[str] = None
    _B2: typing.Optional[str] = None

    _ATTRIBUTES = {
        'global': {
            'logfile': '<String>; Provide a path in your filesystem for the application to log.',
            'debug': 'Set to true to enable debug output.',
            'statefile': '<String> Provide a path in your filesystem for the application to keep a rudimental state.',
            'hostalias': '<String> Provide a hostname',
            'restore': {
                'dir': '<String> Provide a path in your filesystem for the application to restore',
            }
        },
        'borgbackup': {
            'enabled': 'A trigger to activate/deactivate the backup for borgbackup. (Options: True / False).',
            'bin': 'Path to borg binary to use.',
            'repos': [
                {
                    'repo': 'Path to the existing borg repository.',
                    'passphrase': 'Passphrase for the repository',
                    'keep': "For prune actions: How many days to keep in the repository; e.g. 2 for 2 days",
                    'excludes': "A pattern for files to be excluded from the repository; e.g. *github*",
                    'targets': [
                        '/local/path',
                        '/local/path1'
                    ]
                }
            ]
        },
        'restic': {
            'enabled': 'A trigger to activate the backup with restic. (Options: True / False).',
            'bin': 'Path to borg binary to use.',
            'repos': [
                {
                    'repo': '~/backup # Path to the existing restic repository.',
                    'passphrase': 'Passphrase for the repository',
                    'keep': "For prune actions: How many days to keep in the repository; e.g. 3 for 3 days",
                    'excludes': "A pattern for files to be excluded from the repository; e.g. *github*",
                    'targets': [
                        '/local/path',
                        '/local/path1'
                    ]
                }
            ]
        },
        'syncs': [
            {
                'name': 'Give a meaningful name',
                'type': 'Define the technologie to use (options ...)',
                'source': 'A local path to a directory that you want to sync.',
                'target': 'The destination of the sync; might be a local path or maybe a b2 bucket.'
            }
        ],
        'b2': {
            'binary': 'Path to b2 binary to use.',
            'env': [
                {
                    'name': 'B2_APPLICATION_KEY_ID',
                    'value': '<application key_id from backblaze>'
                },
                {
                    'name': 'B2_APPLICATION_KEY',
                    'value': '<b2 app key>'
                }
            ]
        },
        'flows': [{
            'name': 'Give a meaningful name',
            'tasks': [
                'a milbi subcommand like backup,check,prune,sync',
                'a milbi subcommand like backup,check,prune,sync',
            ]
        }]
    }

    def __init__(self, config_file=None):
        if config_file is None:
            config_file = Config.get_required_env_var("CONFIG_FILE")

        config_file = Config.make_path_absolute(config_file)

        assert os.path.exists(config_file)

        Config._CONFIG_FILE = config_file
        with open(config_file, 'r') as f:
            Config._CONFIG = yaml.safe_load(f)

        # handle relative paths
        Config._CONFIG['global']['logfile'] = Config.make_path_absolute(Config._CONFIG['global']['logfile'])
        Config._CONFIG['global']['restore']['dir'] = Config.make_path_absolute(Config._CONFIG['global']['restore']['dir'])

    @staticmethod
    def get_config_file() -> str:
        return Config._CONFIG_FILE

    @staticmethod
    def get_required_env_var(envvar: str) -> str:
        if envvar not in os.environ:
            raise Exception("Please set the {envvar} environment variable")
        return os.environ[envvar]

    @staticmethod
    def get_required_config_var(configvar: str) -> str:
        assert Config._CONFIG
        if configvar not in Config._CONFIG:
            raise Exception(f"Please set the {configvar} variable in the config file {Config._CONFIG_FILE}")
        return Config._CONFIG[configvar]

    @staticmethod
    def make_path_absolute(inPath):
        """
        function to make sure a path is an absolute path
        """

        if '~' in inPath:
            inPath = os.path.expanduser(inPath)
        if not os.path.isabs(inPath):
            inPath = os.path.join(os.getcwd(), inPath)
        absPath = inPath

        return absPath

    @classmethod
    def get_some_var(cls) -> str:
        """Example variable that is set in the config file (preferred)"""
        if cls._B2 is None:
            cls._B2 = Config.get_required_config_var('b2')
        return cls._B2



from src.milbi.class_milbi import Milbi

def _prepare():
    return Milbi(config="tests/files/clitest-config.yaml", debug=False)


def test_unit_load_config():
    """
    execute y
    """
    obj = _prepare()

    assert isinstance(obj._load_config('tests/files/clitest-config.yaml'), dict)


#!/usr/bin/env python3
import os
import sys
import shutil
import subprocess
"""
some basic tests that uses the cli
"""


def test_cli_broken():
    """
    test if cli can detect broken commands
    """
    cmd = _prepare()
    cmd['args'].extend(['b0rked'])
    assert _run(cmd) > 0


def test_cli():
    """
    test if cli can run without subcommands
    """
    cmd = _prepare()
    assert _run(cmd) == 0


def test_cli_help():
    """
    test if cli can print help
    """
    cmd = _prepare()
    cmd['args'].extend(['--help'])
    assert _run(cmd) == 0


def test_cli_config():
    """
    test if cli can access the config
    """
    cmd = _prepare()
    cmd['args'].extend(['config'])
    assert _run(cmd) == 0


def test_cli_explain():
    """
    test if cli can show the config explanation
    """
    cmd = _prepare()
    cmd['args'].extend(['config', '--explain'])
    assert _run(cmd) == 0


def test_cli_backup_simulate():
    """
    test if cli can do a simulated backup
    """
    cmd = _prepare()
    cmd['args'].extend(['backup', '--simulate'])
    assert _run(cmd) == 0


def test_cli_backup():
    """
    test if cli can execute a backup
    """
    cmd = _prepare()
    cmd['args'].extend(['backup'])
    assert _run(cmd) == 0


def test_cli_info():
    """
    test if cli can print info
    """
    cmd = _prepare()
    cmd['args'].extend(['info'])
    assert _run(cmd) == 0


def test_cli_check():
    """
    test if cli can execute the consistency checks
    """
    cmd = _prepare()
    cmd['args'].extend(['check'])
    assert _run(cmd) == 0


def test_cli_state():
    """
    test if cli can execute state subcmd
    """
    cmd = _prepare()
    cmd['args'].extend(['state'])
    assert _run(cmd) == 0


def test_cli_prune():
    """
    test if cli can prune from the repository
    """
    cmd = _prepare()
    cmd['args'].extend(['prune'])
    assert _run(cmd) == 0


def test_cli_sync():
    """
    test if cli can execute syncs
    """
    _init()

    cmd = _prepare()
    cmd['args'].extend(['sync'])
    assert _run(cmd) == 0

    _cleanup()


def test_cli_restore():
    """
    test if cli can execute restores
    """
    _init()

    cmd = _prepare()
    cmd['args'].extend(['get', '--repo', 'resticrepo'])
    assert _run(cmd) == 0

    _cleanup()


def test_cli_fuzz():
    """
    test if cli can execute fuzz testing
    """
    _init()

    cmd = _prepare()
    cmd['args'].extend(['fuzz', '--count', '3'])
    assert _run(cmd) == 0

    _cleanup()


def test_cli_flows():
    """
    test if cli can execute all flows
    """
    _init()

    cmd = _prepare()
    cmd['args'].extend(['flows'])
    assert _run(cmd) == 0

    _cleanup()


def test_cli_flow():
    """
    test if cli can execute one specific flow
    """
    _init()

    cmd = _prepare()
    cmd['args'].extend(['flows', '--flow', 'quick'])
    assert _run(cmd) == 0

    cmd = _prepare()
    cmd['args'].extend(['flows', '--flow', 'none'])
    assert _run(cmd) == 0

    cmd = _prepare()
    cmd['args'].extend(['flows', '--flow', 'wrong'])
    assert _run(cmd) == 0

    _cleanup()


"""
helpers
"""

def _init():
    try:
        for item in ['./tests/files/resticrepo-copy', './tests/files/resticrepo-restore']:
            if not os.path.exists(item):
                os.mkdir(item)
    except:
        pass

def _prepare():
    cmd = {
        'args': ['./milbi.py', '--config', './tests/files/clitest-config.yaml'],
    }
    return cmd


def _cleanup():
    for item in ['./tests/files/resticrepo-copy', './tests/files/resticrepo-restore']:
        if os.path.exists(item) and os.path.isdir(item):
            for files in os.listdir(item):
                path = os.path.join(item, files)
                try:
                    shutil.rmtree(path)
                except OSError:
                    os.remove(path)

def _run(cmd):
    try:
        process = subprocess.Popen(
            cmd['args'], shell=False, stdin=None)
    except Exception as e:
        print(f"ERROR running {cmd}")
        print(f"ERROR {format(sys.exc_info()[1])} \n{e}")
        return 1
    while True:
        if process.poll() is not None:
            break
    rc = process.poll()
    return rc



#!/usr/bin/env python3

import os
import json
import typing


class State(object):
    _state: typing.Optional[dict] = {
        'current': {},
        'previous': {},
    }
    _state_file = None
    _read_only = True

    def __init__(self, state_file=None, read_only=True):

        self._read_only = read_only
        self._state_file = state_file

        if not os.path.exists(state_file):
            open(state_file, 'a').close()

        with open(state_file) as f:
            try:
                old_one = json.load(f)
                self._state['previous'] = old_one['current']
            except Exception:
                self._state['previous'] = '{ }'

    def __del__(self):
        if not self._read_only:
            file_handle = open(self._state_file, "w")
            file_handle.write(f"{json.dumps(self._state)}")
            file_handle.close()

    def set_state_readwrite(self) -> bool:
        self._read_only = False
        return self._read_only

    def get_state(self) -> str:
        return self._state

    def set_state(self, to_add) -> bool:
        self._state['current'].update(to_add)

        return True
